/* ###################################################################
**     Filename    : main.c
**     Processor   : S32K1xx
**     Abstract    :
**         Main module.
**         This module contains user's application code.
**     Settings    :
**     Contents    :
**         No public methods
**
** ###################################################################*/
/*!
** @file main.c
** @version 01.00
** @brief
**         Main module.
**         This module contains user's application code.
*/         
/*!
**  @addtogroup main_module main module documentation
**  @{
*/         
/* MODULE main */

/* Including necessary module. Cpu.h contains other modules needed for compiling.*/
#include "Cpu.h"

  volatile int exit_code = 0;

/* User includes (#include below this line is not maintained by Processor Expert) */

#include "Config.h"

/*! 
  \brief The main function for the project.
  \details The startup initialization sequence is the following:
 * - startup asm routine
 * - main()
*/
int main(void)
{
  /* Write your local variable definition here */

  /*** Processor Expert internal initialization. DON'T REMOVE THIS CODE!!! ***/
  #ifdef PEX_RTOS_INIT
    PEX_RTOS_INIT();                   /* Initialization of the selected RTOS. Macro is defined by the RTOS component. */
  #endif
  /*** End of Processor Expert internal initialization.                    ***/

  /* Write your code here */
  /* For example: for(;;) { } */

    /* Initialize and configure clocks */
    CLOCK_SYS_Init(g_clockManConfigsArr, CLOCK_MANAGER_CONFIG_CNT,
                        g_clockManCallbacksArr, CLOCK_MANAGER_CALLBACK_CNT);
    CLOCK_SYS_UpdateConfiguration(0U, CLOCK_MANAGER_POLICY_AGREEMENT);

    /* Initialize pins */
    PINS_DRV_Init(NUM_OF_CONFIGURED_PINS, g_pin_mux_InitConfigArr);

    CAN_User_Init();/* initialize the CAN module as needed */

    /* start the RxFIFO CAN message receive */
    FLEXCAN_DRV_RxFifo(INST_CANCOM1,&CAN_RxFIFO_RX_Message_Buffer[CAN_RxFIFO_RX_BufferIndex]);

   /* start the individual MB CAN message receive */
    FLEXCAN_DRV_Receive(INST_CANCOM1, RX_MB, &CAN_RX_Message_Buffer[CAN_RX_BufferIndex]);

    while(1)
	{
		/* check whether CAN RX is completed, and feedback the received CAN message */
		if(true==CAN_RX_Completed_Flag)
		{
			CAN_RX_Completed_Flag = false;/* clean the flag */

			/* the new receive data stored in previous RX buffer, get its index */
			CAN_RxDataBufferIndex = (0==CAN_RX_BufferIndex) ? (CAN_RX_Buffer_MAX - 1) : (CAN_RX_BufferIndex - 1);

			/* update the TX MB configuration according to the received message */
			CAN_MB_Config_Update(&CAN_TX_MB_Config, CAN_RX_Message_Buffer[CAN_RxDataBufferIndex].cs);

			/*get the data length */
			CAN_TX_MB_Config.data_length = CAN_RX_Message_Buffer[CAN_RxDataBufferIndex].dataLen;

			/* send it back via CAN */
			FLEXCAN_DRV_SendBlocking(INST_CANCOM1,TX_MB, &CAN_TX_MB_Config,\
					CAN_RX_Message_Buffer[CAN_RxDataBufferIndex].msgId,\
					&CAN_RX_Message_Buffer[CAN_RxDataBufferIndex].data[0], \
					1000);

			/* toggle the blue RGB LED */
			PINS_DRV_TogglePins(RGB_LED_GPIO,(1<<RGB_BLUE_LED_PIN));
		}

		/* check whether CAN RxFIFO RX is completed, and feedback the received CAN message */
		if(true==CAN_RxFIFO_RX_Completed_Flag)
		{
			CAN_RxFIFO_RX_Completed_Flag = false;/* clean the flag */

			/* the new receive data stored in previous RX buffer, get its index */
			CAN_RxFIFO_RxDataBufferIndex = (0==CAN_RxFIFO_RX_BufferIndex) ? (CAN_RxFIFO_RX_Buffer_MAX - 1 ) : (CAN_RxFIFO_RX_BufferIndex - 1);

			/* update the TX MB configuration according to the received message */
			CAN_MB_Config_Update(&CAN_TX_MB_Config, CAN_RxFIFO_RX_Message_Buffer[CAN_RxFIFO_RxDataBufferIndex].cs);

			/*get the data length */
			CAN_TX_MB_Config.data_length = CAN_RxFIFO_RX_Message_Buffer[CAN_RxFIFO_RxDataBufferIndex].dataLen;

			/* send it back via CAN */
			FLEXCAN_DRV_SendBlocking(INST_CANCOM1,TX_MB, &CAN_TX_MB_Config,\
					CAN_RxFIFO_RX_Message_Buffer[CAN_RxFIFO_RxDataBufferIndex].msgId,\
					&CAN_RxFIFO_RX_Message_Buffer[CAN_RxFIFO_RxDataBufferIndex].data[0], \
					1000);

			/* toggle the red RGB LED */
			PINS_DRV_TogglePins(RGB_LED_GPIO,(1<<RGB_RED_LED_PIN));
		}

		CAN_TX_MB_Config.msg_id_type = FLEXCAN_MSG_ID_STD; /* set the TX message type */
		CAN_TX_MB_Config.data_length = 8; /* set TX data length */
		FLEXCAN_DRV_SendBlocking(INST_CANCOM1,TX_MB,&CAN_TX_MB_Config,TX_MSG_ID,can_msg_data,1000);

		(*(uint64_t *)&can_msg_data[0])++;/* increase the data */

		OSIF_TimeDelay(5);/* delay 5ms */
	}

  /*** Don't write any code pass this line, or it will be deleted during code generation. ***/
  /*** RTOS startup code. Macro PEX_RTOS_START is defined by the RTOS component. DON'T MODIFY THIS CODE!!! ***/
  #ifdef PEX_RTOS_START
    PEX_RTOS_START();                  /* Startup of the selected RTOS. Macro is defined by the RTOS component. */
  #endif
  /*** End of RTOS startup code.  ***/
  /*** Processor Expert end of main routine. DON'T MODIFY THIS CODE!!! ***/
  for(;;) {
    if(exit_code != 0) {
      break;
    }
  }
  return exit_code;
  /*** Processor Expert end of main routine. DON'T WRITE CODE BELOW!!! ***/
} /*** End of main routine. DO NOT MODIFY THIS TEXT!!! ***/

/* END main */
/*!
** @}
*/
/*
** ###################################################################
**
**     This file was created by Processor Expert 10.1 [05.21]
**     for the NXP S32K series of microcontrollers.
**
** ###################################################################
*/
